package com.jpattern.orm.query.from;

import com.jpattern.orm.classtool.IOrmClassToolMap;
import com.jpattern.orm.query.INameSolver;

/**
 * 
 * @author Francesco Cina
 *
 * 27/giu/2011
 */
public abstract class AFromElement implements FromElement {

	protected final Class<?> joinClass;
	private INameSolver nameSolver;
	protected final IOrmClassToolMap ormClassToolMap;
	private final Integer nameSolverClassId;

	public AFromElement(final IOrmClassToolMap ormClassToolMap, final Class<?> joinClass, final Integer nameSolverClassId) {
		this.ormClassToolMap = ormClassToolMap;
		this.joinClass = joinClass;
		this.nameSolverClassId = nameSolverClassId;
	}

	@Override
	public void setNameSolver(final INameSolver nameSolver) {
		this.nameSolver = nameSolver;
	}

	@Override
	public String renderSql() {
		StringBuilder query = new StringBuilder();
		renderSql(query);
		return query.toString();
	}

	@Override
	public void renderSql(final StringBuilder StringBuilder) {
		String alias = this.nameSolver.alias(getNameSolverClassId());
		StringBuilder.append( getJoinName() );
		StringBuilder.append(this.ormClassToolMap.getOrmClassTool(this.joinClass).getClassMapper().getTableMap().getTableNameWithSchema() );
		StringBuilder.append( " " );
		StringBuilder.append(alias);

		if (hasOnClause()) {
			StringBuilder.append( " ON " );
			StringBuilder.append( this.nameSolver.solvePropertyName(onLeftProperty()) );
			StringBuilder.append( " = " );
			StringBuilder.append( this.nameSolver.solvePropertyName(onRightProperty()) );
		}

		StringBuilder.append( " " );
	}

	protected abstract String getJoinName();

	protected abstract boolean hasOnClause();

	protected abstract String onLeftProperty();

	protected abstract String onRightProperty();

	public Integer getNameSolverClassId() {
		return this.nameSolverClassId;
	}

}